@@ -9,11 +9,12 @@ static const char* ecasound_parameter_help_rep =
 "     -d:debug_level           show debug info \n"
 "     -D                       print all debug info to stderr\n"
 "     -q                       quiet mode, no output \n"
-"     -s[:]file                load chainsetup from 'file' \n"
+"     -s[:]file.ecs            load chainsetup from 'file' \n"
+"     -E \"foo1 ; foo 2\"      execute interactive commands at start\n"
 "     --daemon                 enable ecasound's daemon mode\n"
 "     --daemon-port=XXX        use TCP port XXX\n"
 "     --nodaemon               disable the daemon mode (default)\n"
-"     --keep-running (or -K)   no not exit from batchmode\n"
+"     --keep-running (or -K)   do not exit from batchmode\n"
 "     --help (or -h)           show this help\n"
 "     --version                print version info\n"
 " --- \n"
