@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-curses.cpp: Curses implementation of the console user interface.
-// Copyright (C) 1999-2004 Kai Vehmanen
+// Copyright (C) 1999-2004,2007 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 2
@@ -24,6 +24,7 @@
 #include <config.h>
 #endif
 
+#include <cstdlib>
 #include <iostream>
 #include <map>
 #include <string>
@@ -131,9 +132,6 @@ void ECA_CURSES::init_readline_support(void)
 /*                                                                  */
 /* **************************************************************** */
 
-char *command_generator ();
-char **fileman_completion ();
-
 /**
  * Attempt to complete on the contents of TEXT.  START and END bound the
  * region of rl_line_buffer that contains the word to complete.  TEXT is
