@@ -31,6 +31,7 @@
 #include <netinet/in.h>   /* BSD: inet_ntoa() */
 #include <sys/poll.h>     /* POSIX: poll() */
 #include <sys/socket.h>   /* BSD: getpeername() */
+#include <sys/types.h>    /* OSX: u_int32_t (INADDR_ANY) */
 
 #include <kvu_dbc.h>
 #include <kvu_fd_io.h>
@@ -193,9 +194,9 @@ void ECA_NETECI_SERVER::open_server_socket(void)
     std::cerr << "bind() failed." << endl;
   }
   
-  DBC_ENSURE((unix_sockets_rep == true) && 
-	     ((server_listening_rep == true && socketpath_rep.size() > 0 ||
-	       server_listening_rep != true && socketpath_rep.size() == 0)) ||
+  DBC_ENSURE((unix_sockets_rep == true && 
+	     (((server_listening_rep == true && socketpath_rep.size() > 0) ||
+	       (server_listening_rep != true && socketpath_rep.size() == 0)))) ||
 	     (unix_sockets_rep != true));
 }
 
