@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // ecasound.cpp: Console mode user interface to ecasound.
-// Copyright (C) 2002-2007 Kai Vehmanen
+// Copyright (C) 2002-2008 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3 (see Ecasound Programmer's Guide)
@@ -56,6 +56,7 @@
 
 static void ecasound_create_eca_objects(struct ecasound_state* state, COMMAND_LINE& cline);
 static void ecasound_launch_daemon(struct ecasound_state* state);
+static int ecasound_pass_at_launch_commands(struct ecasound_state* state);
 static void ecasound_main_loop(struct ecasound_state* state);
 void ecasound_parse_command_line(struct ecasound_state* state, 
 				 const COMMAND_LINE& clinein,
@@ -65,8 +66,8 @@ static void ecasound_print_version_banner(void);
 static void ecasound_setup_signals(struct ecasound_state* state);
 
 extern "C" {
-  static void ecasound_atexit_cleanup(void);
-  static void* ecasound_watchdog_thread(void* arg); 
+  static void ecasound_exit_cleanup(void);
+  static void* ecasound_signal_watchdog_thread(void* arg); 
   static void ecasound_signal_handler(int signal);
 }
 
@@ -76,6 +77,7 @@ static struct ecasound_state ecasound_state_global =
     0,        /* ECA_LOGGER_INTERFACE */
     0,        /* ECA_NETECI_SERVER */
     0,        /* ECA_SESSION */
+    0,        /* launchcmds, std::vector<std::string> */
     0,        /* pthread_t - daemon_thread */
     0,        /* pthread_mutex_t - lock */
     0,        /* sig_wait_t - exit_request */
@@ -91,6 +93,7 @@ static struct ecasound_state ecasound_state_global =
 
 static sig_atomic_t ecasound_cleanup_done = 0;
 static sig_atomic_t ecasound_normal_exit = 0;
+static sig_atomic_t ecasound_watchdog_active = 0;
 
 /**
  * Namespace imports
@@ -104,7 +107,7 @@ int main(int argc, char *argv[])
 {
   struct ecasound_state* state = &ecasound_state_global;
 
-  /* 1. setup signals and watchdog thread */
+  /* 1. setup signals and the signal watchdog thread */
   ecasound_setup_signals(state);
 
   /* 2. parse command-line args */
@@ -153,7 +156,10 @@ int main(int argc, char *argv[])
       }
     }
 
-    /* 8. start processing */
+    /* 8. pass launch commands */
+    ecasound_pass_at_launch_commands(state);
+
+    /* 9. start processing */
     if (state->retval == ECASOUND_RETVAL_SUCCESS) {
       ecasound_main_loop(state);
     }
@@ -173,13 +179,18 @@ int main(int argc, char *argv[])
   /* note: we prefer to run the cleanup routines before 
    *       returning from main; there have been problems with
    *       dynamic libraries and libecasound cleanup routines
-   *       when run from atexit handler
+   *       when run from an atexit() handler
    */
   ecasound_normal_exit = 1;
-  ecasound_atexit_cleanup();
+  ecasound_exit_cleanup();
 
   // cerr << endl << "ecasound: main() exiting..." << endl << endl;
 
+  DBC_CHECK(state->retval == ECASOUND_RETVAL_SUCCESS ||
+	    state->retval == ECASOUND_RETVAL_INIT_FAILURE ||
+	    state->retval == ECASOUND_RETVAL_START_ERROR ||
+	    state->retval == ECASOUND_RETVAL_RUNTIME_ERROR);
+
   return state->retval;
 }
 
@@ -187,7 +198,7 @@ int main(int argc, char *argv[])
  * Cleanup routine that is run after either exit()
  * is called or ecasound returns from its main().
  */
-void ecasound_atexit_cleanup(void)
+void ecasound_exit_cleanup(void)
 {
   struct ecasound_state* state = &ecasound_state_global;
 
@@ -204,16 +215,12 @@ void ecasound_atexit_cleanup(void)
       }
     }
 
-    if (ecasound_normal_exit == 1 ||
-	state->interactive_mode == true) {
-      /* note: do not call destructors in the case ECA_CONTROL::run()
-       * could still be running (in non-interactive mode, atexit
-       * called due to a signal)	
-       */
-      if (state->control != 0) { delete state->control; state->control = 0; }
-      if (state->session != 0) { delete state->session; state->session = 0; }
-    }
+    DBC_CHECK(ecasound_normal_exit == 1);
 
+    if (state->control != 0) { delete state->control; state->control = 0; }
+    if (state->session != 0) { delete state->session; state->session = 0; }
+
+    if (state->launchcmds != 0) { delete state->launchcmds; state->launchcmds = 0; }
     if (state->eciserver != 0) { delete state->eciserver; state->eciserver = 0; }
     if (state->console != 0) { delete state->console; state->console = 0; }
     if (state->daemon_thread != 0) { delete state->daemon_thread; state->daemon_thread = 0; }
@@ -221,7 +228,7 @@ void ecasound_atexit_cleanup(void)
     if (state->signalset != 0) { delete state->signalset; state->signalset = 0; }
   }
 
-  // cerr << "ecasound: atexit cleanup done." << endl << endl;
+  // cerr << "ecasound: exit cleanup done." << endl << endl;
 }
 
 /**
@@ -277,6 +284,21 @@ void ecasound_launch_daemon(struct ecasound_state* state)
   // state->console->print("ecasound: NetECI server started");
 }
 
+static int ecasound_pass_at_launch_commands(struct ecasound_state* state)
+{
+  if (state->launchcmds) {
+    std::vector<std::string>::const_iterator p = state->launchcmds->begin();
+
+    while(p != state->launchcmds->end()) {
+      state->control->command(*p);
+      state->control->print_last_value();
+      ++p;
+    }
+  }
+
+  return 0;
+}
+
 /**
  * The main processing loop.
  */
@@ -328,10 +350,12 @@ void ecasound_main_loop(struct ecasound_state* state)
     }
 
     if (ctrl->is_connected() == true) {
-      int res = ctrl->run(!state->keep_running_mode);
-      if (res < 0) {
-	state->retval = ECASOUND_RETVAL_RUNTIME_ERROR;
-	cerr << "ecasound: Warning! Errors detected during processing." << endl;
+      if (!state->exit_request) {
+	int res = ctrl->run(!state->keep_running_mode);
+	if (res < 0) {
+	  state->retval = ECASOUND_RETVAL_RUNTIME_ERROR;
+	  cerr << "ecasound: Warning! Errors detected during processing." << endl;
+	}
       }
     }
     else {
@@ -387,6 +411,15 @@ void ecasound_parse_command_line(struct ecasound_state* state,
 	state->daemon_mode = true;
       }
 
+      else if (cline.current().find("-E") != string::npos) {
+	cline.next();
+	if (cline.end() != true) {
+	  state->launchcmds = 
+	    new std::vector<std::string>
+ 	      (kvu_string_to_vector(cline.current(), ';'));
+	}
+      }
+
       else if (cline.current().find("--daemon-port") != string::npos) {
 	std::vector<std::string> argpair = 
 	  kvu_string_to_vector(cline.current(), '=');
@@ -396,6 +429,7 @@ void ecasound_parse_command_line(struct ecasound_state* state,
 	}
       }
 
+
       else if (cline.current() == "--nodaemon") {
 	state->daemon_mode = false;
       }
@@ -436,7 +470,7 @@ void ecasound_print_usage(void)
 void ecasound_print_version_banner(void)
 {
   cout << "ecasound v" << ecasound_library_version << endl;
-  cout << "Copyright (C) 1997-2007 Kai Vehmanen and others." << endl;
+  cout << "Copyright (C) 1997-2008 Kai Vehmanen and others." << endl;
   cout << "Ecasound comes with ABSOLUTELY NO WARRANTY." << endl;
   cout << "You may redistribute copies of ecasound under the terms of the GNU" << endl;
   cout << "General Public License. For more information about these matters, see" << endl; 
@@ -445,45 +479,60 @@ void ecasound_print_version_banner(void)
 
 static void ecasound_signal_handler(int signal)
 {
-#ifdef HAVE_SIGWAIT
-  cerr << "(ecasound-watchdog) WARNING! ecasound_signal_handler entered, this should _NOT_ happen!";
-  cerr << " pid=" << getpid() << endl;
+#if defined(HAVE_SIGPROCMASK) || defined(HAVE_PTHREAD_SIGMASK)
+  if (ecasound_watchdog_active &&
+      ecasound_state_global.exit_request == 0) {
+    cerr << "(ecasound-watchdog) WARNING: ecasound_signal_handler entered, this should _NOT_ happen!";
+    cerr << " pid=" << getpid() << endl;
+  }
 #endif
+
+  if (ecasound_watchdog_active &&
+      ecasound_state_global.exit_request) {
+    cerr << "(ecasound-watchdog) WARNING: Signal received during cleanup, exiting immediately.\n";
+    exit(ECASOUND_RETVAL_RUNTIME_ERROR);
+  }
 }
 
 /**
  * Sets up a signal mask with sigaction() that blocks 
- * all common signals, and then launces an watchdog
+ * all common signals, and then launces a signal watchdog
  * thread that waits on the blocked signals using
- * sigwait().
+ * sigwait(). 
+ * 
+ * This design causes all non-fatal termination signals
+ * to be routed through a single thread. This signal watchdog
+ * in turn performs a clean exit upon receiving a signal.
+ * Without this setup, interactions between threads when handling 
+ * would be harder to control (especially considering that
+ * ecasound needs to work on various different platforms).
  */
 void ecasound_setup_signals(struct ecasound_state* state)
 {
   pthread_t watchdog;
-  sigset_t* signalset = new sigset_t;
-
-  state->signalset = signalset;
+  sigset_t* signalset;
 
   /* man pthread_sigmask:
    *  "...signal actions and signal handlers, as set with
    *   sigaction(2), are shared between all threads"
    */
 
-  /* create a dummy signal handler */
-  struct sigaction blockaction;
-  blockaction.sa_handler = ecasound_signal_handler;
-  sigemptyset(&blockaction.sa_mask);
-  blockaction.sa_flags = 0;
-
-  sigemptyset(signalset);
-
   /* handle the following signals explicitly */
+  signalset = new sigset_t;
+  state->signalset = signalset;
+  sigemptyset(signalset);
   sigaddset(signalset, SIGTERM);
   sigaddset(signalset, SIGINT);
   sigaddset(signalset, SIGHUP);
   sigaddset(signalset, SIGPIPE);
   sigaddset(signalset, SIGQUIT);
 
+  /* create a dummy signal handler */
+  struct sigaction blockaction;
+  blockaction.sa_handler = ecasound_signal_handler;
+  sigemptyset(&blockaction.sa_mask);
+  blockaction.sa_flags = 0;
+
   /* attach the dummy handler to the following signals */
   sigaction(SIGTERM, &blockaction, 0);
   sigaction(SIGINT, &blockaction, 0);
@@ -492,62 +541,125 @@ void ecasound_setup_signals(struct ecasound_state* state)
   sigaction(SIGQUIT, &blockaction, 0);
 
 #ifdef __FreeBSD__
+  /* ignore signals instead of passing them to our handler */
   blockaction.sa_handler = SIG_IGN;
   sigaction(SIGFPE, &blockaction, 0);
 #endif
 
   int res = pthread_create(&watchdog, 
 			   NULL, 
-			   ecasound_watchdog_thread, 
+			   ecasound_signal_watchdog_thread, 
 			   reinterpret_cast<void*>(state));
   if (res != 0) {
     cerr << "ecasound: Warning! Unable to create watchdog thread." << endl;
   }
 
-#ifdef HAVE_SIGPROCMASK
-  /* block all signals */
+  /* block all signals in 'signalset' (see above) */
+#if defined(HAVE_PTHREAD_SIGMASK)
+  pthread_sigmask(SIG_BLOCK, signalset, NULL);
+#elif defined(HAVE_SIGPROCMASK)
   sigprocmask(SIG_BLOCK, signalset, NULL);
 #endif
 }
 
-void* ecasound_watchdog_thread(void* arg)
+/**
+ * Runs a watchdog thread that centrally catches signals that
+ * will cause ecasound to exit.
+ */
+void* ecasound_signal_watchdog_thread(void* arg)
 {
   int signalno = 0;
   struct ecasound_state* state = reinterpret_cast<struct ecasound_state*>(arg);
 
   // cerr << "Watchdog-thread created, pid=" << getpid() << "." << endl;
 
-  /* register cleanup routine */
-  atexit(&ecasound_atexit_cleanup);
-
-  /* 1. block until a signal received */
 #ifdef HAVE_SIGWAIT
-#ifdef HAVE_SIGPROCMASK
+
+  /* step: block execution until a signal received 
+   **********************************************/
+
+  ecasound_watchdog_active = 1;
+
+# if defined(HAVE_PTHREAD_SIGMASK)
+  pthread_sigmask(SIG_BLOCK, state->signalset, NULL);
+# elif defined(HAVE_SIGPROCMASK)
   /* the set of signals must be blocked before entering sigwait() */
-  sigprocmask(SIG_SETMASK, state->signalset, NULL);
-#endif
+  sigprocmask(SIG_BLOCK, state->signalset, NULL);
+# endif
+
   sigwait(state->signalset, &signalno);
-  // cerr << endl << "(ecasound-watchdog) Received signal " << signalno << ". Cleaning up and exiting..." << endl;
 
-  /* 2. use pause() instead */
-#elif HAVE_PAUSE
+  cerr << endl << "(ecasound-watchdog) Received signal " << signalno << ". Cleaning up and exiting..." << endl;
+
+  /* step: use pause() instead (alternative to sigwait()) 
+  ********************************************************/
+
+#elif HAVE_PAUSE /* !HAVE_SIGWAIT */
+
+  /* note: with pause() we don't set 'ecasound_watchdog_active' as
+   * it is normal to get signals when watchdog is running */
   pause();
-  // cerr << endl << "(ecasound-watchdog) Received signal and returned from pause(). Cleaning up and exiting..." << endl;
 
-  /* 3. if proper signal handling is not possible, stop compilation */
-#else
-#error "Neither sigwait() or pause() is available, unable to continue."
+  cerr << endl << "(ecasound-watchdog) Received signal and returned from pause(). Cleaning up and Exiting..." << Endl;
+
+#else /* !HAVE_PAUSE */
+
+  /* 2. If proper signal handling is not possible, stop compilation.
+   *****************************************************************/
+
+# error "Neither sigwait() or pause() Is available, unable to continue."
+
 #endif
 
+  /* step: signal the mainloop that process should terminate */
   state->exit_request = 1;
-  
-  DBC_CHECK(state->retval == ECASOUND_RETVAL_SUCCESS ||
-	    state->retval == ECASOUND_RETVAL_INIT_FAILURE ||
-	    state->retval == ECASOUND_RETVAL_START_ERROR ||
-	    state->retval == ECASOUND_RETVAL_RUNTIME_ERROR);
-  
-  exit(state->retval);
 
-  /* to keep the compilers happy; never actually executed */
+  /* step: unblock signals after process termination has 
+   *       been started */
+
+#ifdef HAVE_SIGWAIT
+# if defined(HAVE_PTHREAD_SIGMASK)
+  pthread_sigmask(SIG_UNBLOCK, state->signalset, NULL);
+# elif defined(HAVE_SIGPROCMASK)
+  /* the set of signals must be blocked before entering sigwait() */
+  sigprocmask(SIG_UNBLOCK, state->signalset, NULL);
+# endif
+#endif
+
+  /* step: in case mainloop is blocked running a batch job, we signal
+   *       the engine thread directly and force it to terminate */
+  if (state->interactive_mode != true &&
+      state->control)
+    state->control->quit_async();
+
+  while(ecasound_normal_exit != 1) {
+    /* sleep for one 200ms */
+    kvu_sleep(0, 200000000);
+
+    /* note: A race condition exists between ECA_CONTROL_BASE
+     *       quit_async() and run(): if quit_async() is called
+     *       after run() has been entered, but before run()
+     *       has managed to start the engine, it is possible engine
+     *       may still be started. 
+     * 
+     *       Thus we will keep checking the engine status until 
+     *       shutdown is really completed. 
+     *
+     *       For robustness, this check is also done when in
+     *       interactive mode (in case the mainloop does not for
+     *       some reason react to our exit request).
+     */
+    if (state->control) {
+      if (state->control->is_engine_started() == true) {
+	state->control->quit_async();
+      }
+    }
+
+  }
+    
+  ecasound_watchdog_active = 0;
+
+  // cerr << endl << "ecasound: watchdog thread exiting..." << endl;
+
   return 0;
 }
