@@ -1,6 +1,8 @@
 #ifndef INCLUDED_ECASOUND_H
 #define INCLUDED_ECASOUND_H
 
+#include <string>
+#include <vector>
 
 #include <pthread.h>   /* POSIX: pthread_create() */
 #include <signal.h>    /* POSIX: sigaction(), sigwait(), sig_atomic_t */
@@ -16,7 +18,7 @@ class ECA_SESSION;
  */
 
 #define ECASOUND_BANNER_ASTERISK_BAR "********************************************************************************\n"
-#define ECASOUND_COPYRIGHT           " (C) 1997-2007 Kai Vehmanen and others    "
+#define ECASOUND_COPYRIGHT           " (C) 1997-2008 Kai Vehmanen and others    "
 
 #define ECASOUND_RETVAL_SUCCESS         0    /**< Succesful run */
 #define ECASOUND_RETVAL_INIT_FAILURE    1    /**< Unable to initialize */
@@ -36,6 +38,7 @@ struct ecasound_state {
   ECA_LOGGER_INTERFACE* logger;
   ECA_NETECI_SERVER* eciserver;
   ECA_SESSION* session;
+  std::vector<std::string>* launchcmds;
   pthread_t* daemon_thread;
   pthread_mutex_t* lock;
   sig_atomic_t exit_request;
