@@ -1,9 +1,9 @@
 // ------------------------------------------------------------------------
 // audioio-faad.cpp: Interface class for FAAC/FAAD AAC encoder/decoder.
-// Copyright (C) 2004-2006 Kai Vehmanen
+// Copyright (C) 2004-2006,2008 Kai Vehmanen
 //
 // Attributes:
-//     eca-style-version: 3
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -78,14 +78,14 @@ void AAC_FORKED_INTERFACE::open(void) throw (AUDIO_IO::SETUP_ERROR &)
       }
       else {
 	urlprefix = std::string(label(), 0, offset);
-	ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audioio-aac) Found url; protocol '" + urlprefix + "'.");
+	ECA_LOG_MSG(ECA_LOGGER::user_objects, "Found url; protocol '" + urlprefix + "'.");
       }
     }
 
     /* decoder supports: fixed channel count and sample format, 
                          sample rate unknown and must be set manually  */
     set_channels(2); /* 5.1 downmixed to stereo if needed by faad2 */
-    /* do not commit: */
+    /* FIXME: fix properly */
     set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_le);
   }
   else {
@@ -111,6 +111,7 @@ void AAC_FORKED_INTERFACE::close(void)
 long int AAC_FORKED_INTERFACE::read_samples(void* target_buffer, long int samples)
 {
   if (triggered_rep != true) { 
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context");
     triggered_rep = true;
     fork_input_process();
   }
@@ -124,7 +125,7 @@ long int AAC_FORKED_INTERFACE::read_samples(void* target_buffer, long int sample
 
   if (bytes_rep < samples * frame_size() || bytes_rep == 0) {
     if (position_in_samples() == 0) 
-      ECA_LOG_MSG(ECA_LOGGER::info, "(audioio-aac) Can't start process \"" + AAC_FORKED_INTERFACE::default_input_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Can't start process \"" + AAC_FORKED_INTERFACE::default_input_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
     finished_rep = true;
     triggered_rep = false;
   }
@@ -137,6 +138,7 @@ long int AAC_FORKED_INTERFACE::read_samples(void* target_buffer, long int sample
 void AAC_FORKED_INTERFACE::write_samples(void* target_buffer, long int samples)
 {
   if (triggered_rep != true) {
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context");
     triggered_rep = true;
     fork_output_process();
   }
@@ -144,6 +146,7 @@ void AAC_FORKED_INTERFACE::write_samples(void* target_buffer, long int samples)
   if (wait_for_child() != true) {
     finished_rep = true;
     triggered_rep = false;
+    ECA_LOG_MSG(ECA_LOGGER::errors, "Attempt to write after child process has terminated.");
   }
   else {
     if (filedes_rep > 0) {
@@ -152,25 +155,22 @@ void AAC_FORKED_INTERFACE::write_samples(void* target_buffer, long int samples)
     else {
       bytes_rep = 0;
     }
-    if (bytes_rep < frame_size() * samples || bytes_rep == 0) {
-      ECA_LOG_MSG(ECA_LOGGER::info, "(audioio-aac) Can't start process \"" + AAC_FORKED_INTERFACE::default_output_cmd + "\". FAAC v1.24 or newer is required. Please check your ~/.ecasound/ecasoundrc.");
+    if (bytes_rep < frame_size() * samples) {
+      ECA_LOG_MSG(ECA_LOGGER::info, "Can't start process \"" + AAC_FORKED_INTERFACE::default_output_cmd + "\". FAAC v1.24 or newer is required. Please check your ~/.ecasound/ecasoundrc.");
       finished_rep = true;
       triggered_rep = false;
+      ECA_LOG_MSG(ECA_LOGGER::errors, 
+		  "Error in writing to child process (to write " 
+		  + kvu_numtostr(frame_size() * samples) 
+		  + ", result "
+		  + kvu_numtostr(bytes_rep) 
+		  + ").");
     }
-    else finished_rep = false;
+    else 
+      finished_rep = false;
   }
 }
 
-void AAC_FORKED_INTERFACE::seek_position(void)
-{
-  if (pid_of_child() > 0) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audioio-aac) Cleaning child process pid=" + kvu_numtostr(pid_of_child()) + ".");
-    clean_child();
-    triggered_rep = false;
-  }
-  set_position_in_samples(0);
-}
-
 void AAC_FORKED_INTERFACE::set_parameter(int param, string value)
 {
   switch (param) {
@@ -230,3 +230,27 @@ void AAC_FORKED_INTERFACE::fork_output_process(void)
     filedes_rep = 0;
   }
 }
+
+void AAC_FORKED_INTERFACE::start_io(void)
+{
+  if (triggered_rep != true) {
+    if (io_mode() == io_read) 
+      fork_input_process();
+    else
+      fork_output_process();
+
+    triggered_rep = true;
+  }
+}
+
+void AAC_FORKED_INTERFACE::stop_io(void)
+{
+  if (triggered_rep == true) {
+    if (io_mode() == io_read) 
+      clean_child(true);
+    else
+      clean_child(false);
+
+    triggered_rep = false;
+  }
+}
