@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 // audioio-flac.cpp: Interface to FLAC decoders and encoders using UNIX 
 //                   pipe i/o.
-// Copyright (C) 2004-2006 Kai Vehmanen
+// Copyright (C) 2004-2006,2008 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -123,6 +123,7 @@ void FLAC_FORKED_INTERFACE::close(void)
 long int FLAC_FORKED_INTERFACE::read_samples(void* target_buffer, long int samples)
 {
   if (triggered_rep != true) { 
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context");
     triggered_rep = true;
     fork_input_process();
   }
@@ -149,6 +150,7 @@ long int FLAC_FORKED_INTERFACE::read_samples(void* target_buffer, long int sampl
 void FLAC_FORKED_INTERFACE::write_samples(void* target_buffer, long int samples)
 {
   if (triggered_rep != true) {
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context");
     triggered_rep = true;
     fork_output_process();
   }
@@ -156,6 +158,7 @@ void FLAC_FORKED_INTERFACE::write_samples(void* target_buffer, long int samples)
   if (wait_for_child() != true) {
     finished_rep = true;
     triggered_rep = false;
+    ECA_LOG_MSG(ECA_LOGGER::errors, "Attempt to write after child process has terminated.");
   }
   else {
     if (filedes_rep > 0) {
@@ -164,23 +167,21 @@ void FLAC_FORKED_INTERFACE::write_samples(void* target_buffer, long int samples)
     else {
       bytes_rep = 0;
     }
-    if (bytes_rep < frame_size() * samples || bytes_rep == 0) {
+    if (bytes_rep < frame_size() * samples) {
       finished_rep = true;
       triggered_rep = false;
+      ECA_LOG_MSG(ECA_LOGGER::errors, 
+		  "Error in writing to child process (to write " 
+		  + kvu_numtostr(frame_size() * samples) 
+		  + ", result "
+		  + kvu_numtostr(bytes_rep) 
+		  + ").");
     }
-    else finished_rep = false;
+    else 
+      finished_rep = false;
   }
 }
 
-void FLAC_FORKED_INTERFACE::seek_position(void) {
-  if (pid_of_child() > 0) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(audioio-flac) Cleaning child process pid=" + kvu_numtostr(pid_of_child()) + ".");
-    clean_child();
-    triggered_rep = false;
-  }
-  set_position_in_samples(0);
-}
-
 void FLAC_FORKED_INTERFACE::set_parameter(int param, string value)
 {
   switch (param) {
@@ -261,3 +262,27 @@ void FLAC_FORKED_INTERFACE::fork_output_process(void)
     filedes_rep = 0;
   }
 }
+
+void FLAC_FORKED_INTERFACE::start_io(void)
+{
+  if (triggered_rep != true) {
+    if (io_mode() == io_read) 
+      fork_input_process();
+    else
+      fork_output_process();
+
+    triggered_rep = true;
+  }
+}
+
+void FLAC_FORKED_INTERFACE::stop_io(void)
+{
+  if (triggered_rep == true) {
+    if (io_mode() == io_read) 
+      clean_child(true);
+    else
+      clean_child(false);
+
+    triggered_rep = false;
+  }
+}
