@@ -1,10 +1,10 @@
 // ------------------------------------------------------------------------
 // audioio-forked-streams.cpp: Helper class providing routines for
 //                             forking for piped input/output.
-// Copyright (C) 2000-2004,2006 Kai Vehmanen
+// Copyright (C) 2000-2004,2006,2008 Kai Vehmanen
 //
 // Attributes:
-//     eca-style-version: 3
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include <kvu_dbc.h>
 #include <kvu_numtostr.h>
@@ -80,6 +81,18 @@ static int afs_run_exec(const string& command, const string& filename)
   return execvp(temp[0].c_str(), const_cast<char**>(args));
 }
 
+AUDIO_IO_FORKED_STREAM::~AUDIO_IO_FORKED_STREAM(void)
+{
+  if (pid_of_child_rep > 0) 
+    clean_child(true);
+}
+
+void AUDIO_IO_FORKED_STREAM::stop_io(void)
+{
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, "stop_io()");
+  clean_child(false);
+}
+
 /**
  * If found, replaces the string '%f' with 'filename'. This is
  * the file used by the forked child for input/output.
@@ -181,17 +194,25 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_read(void)
   else {
     int fpipes[2];
     if (pipe(fpipes) == 0) {
-#ifdef HAVE_SIGPROCMASK
-      sigset_t oldset, newset;
-      sigemptyset(&newset);
-      sigaddset(&newset, SIGTERM);
-      sigprocmask(SIG_UNBLOCK, &newset, &oldset);
-#endif
+      sigkill_sent_rep = false;
       pid_of_child_rep = fork();
       if (pid_of_child_rep == 0) { 
 	// ---
 	// child 
 	// ---
+
+	sigset_t newset;
+	sigemptyset(&newset);
+
+	sigaddset(&newset, SIGTERM);
+	sigaddset(&newset, SIGPIPE);
+
+#if defined(HAVE_PTHREAD_SIGMASK)
+	pthread_sigmask(SIG_UNBLOCK, &newset, NULL);
+#elif defined(HAVE_SIGPROCMASK)
+	sigprocmask(SIG_UNBLOCK, &newset, NULL);
+#endif
+
 	::close(1);
 	dup2(fpipes[1], 1);
 	::close(fpipes[0]);
@@ -206,9 +227,7 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_read(void)
 	// ---
 	// parent
 	// ---
-#ifdef HAVE_SIGPROCMASK
-	sigprocmask(SIG_SETMASK, &oldset, 0);
-#endif
+
 	pid_of_parent_rep = ::getpid();
 	::close(fpipes[1]);
 	fd_rep = fpipes[0];
@@ -228,17 +247,24 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_fifo_read(void)
 
   ECA_LOG_MSG(ECA_LOGGER::user_objects, "Fork child-for-fifo-read: '" + fork_command() + "'");
 
-#ifdef HAVE_SIGPROCMASK
-  sigset_t oldset, newset;
-  sigemptyset(&newset);
-  sigaddset(&newset, SIGTERM);
-  sigprocmask(SIG_UNBLOCK, &newset, &oldset);
-#endif
+  sigkill_sent_rep = false;
   pid_of_child_rep = fork();
   if (pid_of_child_rep == 0) { 
     // ---
     // child 
     // ---
+
+    sigset_t newset;
+    sigemptyset(&newset);
+    sigaddset(&newset, SIGTERM);
+    sigaddset(&newset, SIGPIPE);
+    
+#if defined(HAVE_PTHREAD_SIGMASK)
+    pthread_sigmask(SIG_UNBLOCK, &newset, NULL);
+#elif defined(HAVE_SIGPROCMASK)
+    sigprocmask(SIG_UNBLOCK, &newset, NULL);
+#endif
+
     freopen("/dev/null", "w", stderr);
     int res = afs_run_exec(command_rep, object_rep);
     if (res < 0) {
@@ -258,9 +284,7 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_fifo_read(void)
     // ---
     // parent
     // ---
-#ifdef HAVE_SIGPROCMASK
-    sigprocmask(SIG_SETMASK, &oldset, 0);
-#endif
+
     pid_of_parent_rep = ::getpid();
     fd_rep = 0;
     if (wait_for_child() == true)
@@ -279,17 +303,22 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_write(void)
   
   int fpipes[2];
   if (pipe(fpipes) == 0) {
-#ifdef HAVE_SIGPROCMASK
-    sigset_t oldset, newset;
-    sigemptyset(&newset);
-    sigaddset(&newset, SIGTERM);
-    sigprocmask(SIG_UNBLOCK, &newset, &oldset);
-#endif
+    sigkill_sent_rep = false;
     pid_of_child_rep = fork();
     if (pid_of_child_rep == 0) { 
       // ---
       // child 
       // ---
+      sigset_t newset;
+      sigaddset(&newset, SIGTERM);
+      sigaddset(&newset, SIGPIPE);
+    
+#if defined(HAVE_PTHREAD_SIGMASK)
+      pthread_sigmask(SIG_UNBLOCK, &newset, NULL);
+#elif defined(HAVE_SIGPROCMASK)
+      sigprocmask(SIG_UNBLOCK, &newset, NULL);
+#endif
+
       ::close(0);
       ::dup2(fpipes[0],0);
       ::close(fpipes[0]);
@@ -302,9 +331,6 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_write(void)
       // ---
       // parent
       // ---
-#ifdef HAVE_SIGPROCMASK
-      sigprocmask(SIG_SETMASK, &oldset, 0);
-#endif
       pid_of_parent_rep = ::getpid();
       ::close(fpipes[0]);
       fd_rep = fpipes[1];
@@ -318,8 +344,19 @@ void AUDIO_IO_FORKED_STREAM::fork_child_for_write(void)
 
 /**
  * Cleans (waits for) the forked child process. Note! This
- * function must be called from the same thread as 
- * fork_child_for_read/write() was called.
+ * function should be called from the same thread as 
+ * fork_child_for_read/write() was called. 
+ * 
+ * In case the function is called from a different thread, 
+ * it attemts to terminate the child anyways, but the child's 
+ *  state is not known exactly when function returns.
+ *
+ * @param force if true, client is terminated with SIGKILL,
+ *              which guarantees that it terminates (but 
+ *              possibly without going through normal 
+ *              exit procedure); should be avoided especially
+ *              for output objects as this may result in 
+ *              data loss
  */
 void AUDIO_IO_FORKED_STREAM::clean_child(bool force)
 {
@@ -331,30 +368,44 @@ void AUDIO_IO_FORKED_STREAM::clean_child(bool force)
     fd_rep = -1;
   }
 
-  if (pid_of_child_rep > 0) {
-    int flags = 0;
-#ifdef __WALL
-    /* Linux-specific flag to waitpid() for to wait for
-     * childs created by other threads */
-    flags += __WALL;
-#endif
-    /* wait until child process has exited */
-    int status = 0;
-    int res = waitpid(pid_of_child_rep, &status, flags);
-    if (res == pid_of_child_rep && WIFEXITED(status)) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Child process exit ok");
-      pid_of_child_rep = 0;
+  if (pid_of_child_rep > 0 && 
+      force == true) {
+    if (sigkill_sent_rep != true) {
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, 
+		  "Sending SIGKILL to child process related to: "
+		  + object_rep);
+      kill(pid_of_child_rep, SIGKILL);
+      sigkill_sent_rep = true;
     }
     else {
-      if (res < 0)
-	perror("waitpid");
+      /* SIGKILL already sent once for this process, don't send it again */
+      pid_of_child_rep = -1;
     }
   }
 
-  /* if 'killing you softly' didn't work, next try with SIGTERM */
-  if (force == true && pid_of_child_rep > 0) {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "sending SIGTERM to child process");
-    /* close did not trigger exit, send SIGTERM */
+  if (pid_of_child_rep > 0 &&
+      pid_of_parent_rep == getpid()) {
+    /* wait until child process has exited 
+     * note: this only works reliable when our pid is
+     *       the same as used for starting the child */
+    int flags = 0;
+    int status = 0;
+    int res = waitpid(pid_of_child_rep, &status, flags);
+
+    if (res == pid_of_child_rep) {
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Child process exit ok: "
+		  + object_rep);
+      pid_of_child_rep = 0;
+    }
+    else {
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Problems in terminating child process:" + std::string(strerror(errno)));
+    }
+  }
+
+  if (pid_of_child_rep > 0) {
+    /* wait didn't work, terminate with SIGTERM to be sure */
+    ECA_LOG_MSG(ECA_LOGGER::system_objects, "Sending SIGTERM to child process: "
+		+ object_rep);
     kill(pid_of_child_rep, SIGTERM);
     pid_of_child_rep = 0;
   }
@@ -371,12 +422,18 @@ void AUDIO_IO_FORKED_STREAM::clean_child(bool force)
  */
 bool AUDIO_IO_FORKED_STREAM::wait_for_child(void) const
 {
-  if (pid_of_child_rep > 0) {
+  if (pid_of_child_rep <= 0) 
+    return false;
+  else if (pid_of_parent_rep == getpid() &&
+	   pid_of_child_rep > 0) {
     int pid = waitpid(pid_of_child_rep, 0, WNOHANG);
-    if (pid > 0) {
+    if (pid == pid_of_child_rep) {
       return false;
     }
-    if (pid == 0) return true;
+    /* no change in state, so still active */
+    return true;
   }
-  return false;
+  else 
+    /* note: we don't really know so assume that yes */
+    return true;
 }
