@@ -4,6 +4,8 @@
 #include <string>
 #include <kvu_temporary_file_directory.h>
 
+#include "audioio-barrier.h"
+
 /**
  * Helper class providing routines for forking new processes 
  * and creating read/write pipes between the child and the 
@@ -11,7 +13,7 @@
  *
  * @author Kai Vehmanen
  */
-class AUDIO_IO_FORKED_STREAM {
+class AUDIO_IO_FORKED_STREAM : public AUDIO_IO_BARRIER {
 
  private:
 
@@ -19,6 +21,7 @@ class AUDIO_IO_FORKED_STREAM {
   int pid_of_child_rep;
   int fd_rep;
   bool last_fork_rep;
+  bool sigkill_sent_rep;
   std::string tmpfile_repp;
   bool tmp_file_created_rep;
   bool use_named_pipe_rep;
@@ -29,7 +32,11 @@ class AUDIO_IO_FORKED_STREAM {
   void init_temp_directory(void);
   void fork_child_for_fifo_read(void);
 
- public:
+public:
+
+  virtual void stop_io(void);
+
+ protected:
   
   /**
    * Initializes the command string. This must be done before any other set_* 
@@ -54,12 +61,17 @@ class AUDIO_IO_FORKED_STREAM {
   int pid_of_child(void) const { return(pid_of_child_rep); }
   int file_descriptor(void) const { return(fd_rep); }
 
+public:
+
   AUDIO_IO_FORKED_STREAM(void) : 
-    pid_of_child_rep(0),
+    pid_of_parent_rep(-1),
+    pid_of_child_rep(-1),
     fd_rep(0),
     last_fork_rep(false),
+    sigkill_sent_rep(false),
     tmp_file_created_rep(false),
     use_named_pipe_rep(false) { }
+  virtual ~AUDIO_IO_FORKED_STREAM(void);
 };
 
 #endif
