@@ -14,7 +14,7 @@
  * @author Kai Vehmanen
  */
 class MP3FILE : public AUDIO_IO_BUFFERED,
-		protected AUDIO_IO_FORKED_STREAM {
+		public AUDIO_IO_FORKED_STREAM {
 
  private:
   
@@ -42,6 +42,7 @@ class MP3FILE : public AUDIO_IO_BUFFERED,
 
   virtual int supported_io_modes(void) const { return(io_read | io_write); }
   virtual bool supports_seeking(void) const { return io_mode() == io_read; }
+  virtual bool supports_seeking_sample_accurate(void) const { return false; }
 
   virtual void open(void) throw(AUDIO_IO::SETUP_ERROR &);
   virtual void close(void);
@@ -50,15 +51,14 @@ class MP3FILE : public AUDIO_IO_BUFFERED,
   virtual void write_samples(void* target_buffer, long int samples);
 
   virtual bool finished(void) const { return finished_rep; }
-  virtual void seek_position(void);
+  virtual SAMPLE_SPECS::sample_pos_t seek_position(SAMPLE_SPECS::sample_pos_t pos);
 
   virtual void set_parameter(int param, std::string value);
   virtual std::string get_parameter(int param) const;
 
-  // --
-  // Realtime related functions
-  // --
-  
+  virtual void start_io(void);
+  virtual void stop_io(void);
+
  private:
 
   bool finished_rep;
