@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio-ogg.cpp: Interface for ogg vorbis decoders and encoders.
-// Copyright (C) 2000-2002,2004-2006 Kai Vehmanen
+// Copyright (C) 2000-2002,2004-2006,2008 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3 (see Ecasound Programmer's Guide)
@@ -103,6 +103,7 @@ void OGG_VORBIS_INTERFACE::close(void)
 long int OGG_VORBIS_INTERFACE::read_samples(void* target_buffer, long int samples)
 {
   if (triggered_rep != true) { 
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context"); 
     triggered_rep = true;
     fork_input_process();
   }
@@ -129,6 +130,7 @@ long int OGG_VORBIS_INTERFACE::read_samples(void* target_buffer, long int sample
 void OGG_VORBIS_INTERFACE::write_samples(void* target_buffer, long int samples)
 {
   if (triggered_rep != true) {
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context"); 
     triggered_rep = true;
     fork_output_process();
   }
@@ -136,6 +138,7 @@ void OGG_VORBIS_INTERFACE::write_samples(void* target_buffer, long int samples)
   if (wait_for_child() != true) {
     finished_rep = true;
     triggered_rep = false;
+    ECA_LOG_MSG(ECA_LOGGER::errors, "Attempt to write after child process has terminated.");
   }
   else {
     if (filedes_rep > 0) {
@@ -144,24 +147,21 @@ void OGG_VORBIS_INTERFACE::write_samples(void* target_buffer, long int samples)
     else {
       bytes_rep = 0;
     }
-    if (bytes_rep < frame_size() * samples || bytes_rep == 0) {
+    if (bytes_rep < frame_size() * samples) {
       finished_rep = true;
       triggered_rep = false;
+      ECA_LOG_MSG(ECA_LOGGER::errors, 
+		  "Error in writing to child process (to write " 
+		  + kvu_numtostr(frame_size() * samples) 
+		  + ", result "
+		  + kvu_numtostr(bytes_rep) 
+		  + ").");
     }
-    else finished_rep = false;
+    else 
+      finished_rep = false;
   }
 }
 
-void OGG_VORBIS_INTERFACE::seek_position(void)
-{
-  if (pid_of_child() > 0) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Cleaning child process pid=" + kvu_numtostr(pid_of_child()) + ".");
-    clean_child();
-    triggered_rep = false;
-  }
-  set_position_in_samples(0);
-}
-
 void OGG_VORBIS_INTERFACE::set_parameter(int param, string value)
 {
   switch (param) {
@@ -245,3 +245,30 @@ void OGG_VORBIS_INTERFACE::fork_output_process(void)
     filedes_rep = 0;
   }
 }
+
+void OGG_VORBIS_INTERFACE::start_io(void)
+{
+  if (triggered_rep != true) {
+    if (io_mode() == io_read) 
+      fork_input_process();
+    else
+      fork_output_process();
+
+    triggered_rep = true;
+  }
+}
+
+void OGG_VORBIS_INTERFACE::stop_io(void)
+{
+  if (triggered_rep == true) {
+    if (io_mode() == io_read) {
+      fclose(f1_rep);
+      f1_rep = 0;
+      clean_child(true);
+    }
+    else
+      clean_child(false);
+
+    triggered_rep = false;
+  }
+}
