@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 // audioio-resample.cpp: A proxy class that resamples the child 
 //                       object's data.
-// Copyright (C) 2002-2004 Kai Vehmanen
+// Copyright (C) 2002-2004,2008 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3
@@ -61,30 +61,29 @@ AUDIO_IO_RESAMPLE* AUDIO_IO_RESAMPLE::clone(void) const
 
 void AUDIO_IO_RESAMPLE::open(void) throw(AUDIO_IO::SETUP_ERROR&)
 {
-  ECA_LOG_MSG(ECA_LOGGER::user_objects, "open " + label() + ".");  
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, 
+	      "open " + 
+	      child_params_as_string(1 + AUDIO_IO_RESAMPLE::child_parameter_offset, &params_rep) + ".");  
 
   if (init_rep != true) {
-    AUDIO_IO* tmp = 0;
-    if (params_rep.size() > 1) {
-      /* 2nd-param: audio object name */
-      string& type = params_rep[2];
-      if (type.size() > 0) {
-	tmp = ECA_OBJECT_FACTORY::create_audio_object(type);
-      }
-    }
+    AUDIO_IO* tmp =
+      ECA_OBJECT_FACTORY::create_audio_object(
+        child_params_as_string(1 + AUDIO_IO_RESAMPLE::child_parameter_offset, &params_rep));
 
     /* FIXME: add check for real-time devices, resample does _not_
      *        work with them (rt API not proxied properly)
      */
     
+    DBC_CHECK(tmp != 0);
     if (tmp != 0) {
       set_child(tmp);
     }
 
     int numparams = child()->number_of_params();
     for(int n = 0; n < numparams; n++) {
-      child()->set_parameter(n + 1, get_parameter(n + 3));
-      numparams = child()->number_of_params(); // in case 'n_o_p()' varies
+      child()->set_parameter(n + 1, get_parameter(n + 1 + AUDIO_IO_RESAMPLE::child_parameter_offset));
+      if (child()->variable_params())
+	numparams = child()->number_of_params();
     }
 
     init_rep = true; /* must be set after dyn. parameters */
@@ -108,11 +107,12 @@ void AUDIO_IO_RESAMPLE::open(void) throw(AUDIO_IO::SETUP_ERROR&)
   }
 
   ECA_LOG_MSG(ECA_LOGGER::user_objects, 
-	      "open(); psfactor=" + kvu_numtostr(psfactor_rep) +
+	      "pre-open(); psfactor=" + kvu_numtostr(psfactor_rep) +
 	      ", child_srate=" + kvu_numtostr(child_srate_rep) +
 	      ", srate=" + kvu_numtostr(samples_per_second()) +
 	      ", bsize=" + kvu_numtostr(buffersize()) +
-	      ", c-bsize=" + kvu_numtostr(child_buffersize_rep) + ".");
+	      ", c-bsize=" + kvu_numtostr(child_buffersize_rep) + 
+	      ", child=" + child()->label() + ".");
 
   /* note, we don't use pre_child_open() as 
    * we want to set srate differently */
@@ -123,6 +123,9 @@ void AUDIO_IO_RESAMPLE::open(void) throw(AUDIO_IO::SETUP_ERROR&)
 
   child()->open();
 
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, 
+	      "post-open(); child=" + child()->label() + ".");
+
   /* same for the post processing */ 
   SAMPLE_SPECS::sample_rate_t orig_srate = samples_per_second();
   if (child()->locked_audio_format() == true) {
@@ -143,7 +146,10 @@ void AUDIO_IO_RESAMPLE::open(void) throw(AUDIO_IO::SETUP_ERROR&)
 
 void AUDIO_IO_RESAMPLE::close(void)
 {
-  if (child()->is_open() == true) child()->close();
+  if (child()->is_open() == true) 
+    child()->close();
+
+  init_rep = false;
 
   AUDIO_IO_PROXY::close();
 }
@@ -205,8 +211,9 @@ void AUDIO_IO_RESAMPLE::set_parameter(int param, string value)
   
   sbuf_rep.resample_set_quality(quality_rep);
 
-  if (param > 2 && init_rep == true) {
-    child()->set_parameter(param - 2, value);
+  if (param > AUDIO_IO_RESAMPLE::child_parameter_offset 
+      && init_rep == true) {
+    child()->set_parameter(param - AUDIO_IO_RESAMPLE::child_parameter_offset, value);
   }
 }
 
@@ -216,8 +223,10 @@ string AUDIO_IO_RESAMPLE::get_parameter(int param) const
 		"get_parameter " + label() + ".");
 
   if (param > 0 && param < static_cast<int>(params_rep.size()) + 1) {
-    if (param > 2 && init_rep == true) {
-      params_rep[param - 1] = child()->get_parameter(param - 2);
+    if (param > AUDIO_IO_RESAMPLE::child_parameter_offset 
+	&& init_rep == true) {
+      params_rep[param - 1] = 
+	child()->get_parameter(param - AUDIO_IO_RESAMPLE::child_parameter_offset);
     }
     return params_rep[param - 1];
   }
@@ -225,15 +234,6 @@ string AUDIO_IO_RESAMPLE::get_parameter(int param) const
   return "";
 }
 
-void AUDIO_IO_RESAMPLE::seek_position(void)
-{
-  ECA_LOG_MSG(ECA_LOGGER::user_objects, 
-		"seek_position " + kvu_numtostr(position_in_samples()) + ".");
-  child()->seek_position_in_samples(position_in_samples());
-
-  AUDIO_IO_PROXY::seek_position();
-}
-
 void AUDIO_IO_RESAMPLE::set_audio_format(const ECA_AUDIO_FORMAT& f_str)
 {
   AUDIO_IO::set_audio_format(f_str);
