@@ -39,6 +39,7 @@ class AUDIO_IO_RESAMPLE : public AUDIO_IO_PROXY {
   /** @name Reimplemented functions from DYNAMIC_PARAMETERS<string> */
   /*@{*/
 
+  virtual bool variable_params(void) const { return true; }
   virtual std::string parameter_names(void) const;
   virtual void set_parameter(int param, std::string value);
   virtual std::string get_parameter(int param) const;
@@ -53,20 +54,6 @@ class AUDIO_IO_RESAMPLE : public AUDIO_IO_PROXY {
 
   /*@}*/
 
-  /** @name Reimplemented functions from ECA_AUDIO_POSITION */
-  /*@{*/
-
-  virtual void seek_position(void);
-
-  /* -- not reimplemented 
-   * virtual SAMPLE_SPECS::sample_pos_t position_in_samples(void) const { return(child()->position_in_samples()); }
-   * virtual SAMPLE_SPECS::sample_pos_t length_in_samples(void) const { return(); }
-   * virtual void set_length_in_samples(SAMPLE_SPECS::sample_pos_t pos);
-   * virtual void set_position_in_samples(SAMPLE_SPECS::sample_pos_t pos);
-   */
-
-  /*@}*/
-
   /** @name Reimplemented functions from AUDIO_IO */
   /*@{*/
 
@@ -108,6 +95,8 @@ class AUDIO_IO_RESAMPLE : public AUDIO_IO_PROXY {
   int quality_rep;
   SAMPLE_BUFFER sbuf_rep;
 
+  static const int child_parameter_offset = 2;
+
   AUDIO_IO_RESAMPLE& operator=(const AUDIO_IO_RESAMPLE& x) { return *this; }
   AUDIO_IO_RESAMPLE (const AUDIO_IO_RESAMPLE& x) { }
 
