@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio.cpp: Routines common for all audio IO-devices.
-// Copyright (C) 1999-2004 Kai Vehmanen
+// Copyright (C) 1999-2004,2008 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3
@@ -31,6 +31,14 @@
 #include "audioio.h"
 #include "eca-logger.h"
 
+/**
+ * FIXME notes  (last update 2008-03-11)
+ *
+ * - Modify default implementation of set_label()/label() so
+ *   that is mapped directly to set/get_parameter(1, ...).
+ */
+
+
 const string& AUDIO_IO::SETUP_ERROR::message(void) const { return message_rep; }
 AUDIO_IO::SETUP_ERROR::Error_type AUDIO_IO::SETUP_ERROR::type(void) const { return type_rep; }
 AUDIO_IO::SETUP_ERROR::SETUP_ERROR(AUDIO_IO::SETUP_ERROR::Error_type type, 
@@ -62,6 +70,10 @@ AUDIO_IO::AUDIO_IO(const string& name,
 /**
  * Returns info about supported I/O modes (bitwise-OR)
  *
+ * Note that return value may change after device is
+ * opened (some objects will refine their attributes after
+ * external resources are acquired).
+ *
  * By default, all I/O modes are supported.
  */
 int AUDIO_IO::supported_io_modes(void) const { return (io_read | io_readwrite | io_write); }
@@ -69,17 +81,41 @@ int AUDIO_IO::supported_io_modes(void) const { return (io_read | io_readwrite |
 /**
  * Whether device supports non-blocking I/O mode.
  *
+ * Note that return value may change after device is
+ * opened (some objects will refine their attributes after
+ * external resources are acquired).
+ *
  * By default, nonblocking mode is not supported.
  */
 bool AUDIO_IO::supports_nonblocking_mode(void) const { return false; }
 
 /**
- * Whether device supports non-blocking I/O mode.
+ * Whether device supports seeking.
+ *
+ * Note that return value may change after device is
+ * opened (some objects will refine their attributes after
+ * external resources are acquired).
  *
  * By default, seeking is supported.
+ *
+ * @see supports_seeking_sample_accurate()
  */
 bool AUDIO_IO::supports_seeking(void) const { return true; }
 
+/**
+ * Whether device supports sample accurate seeking.
+ *
+ * @see supports_seeking()
+ *
+ * Note that return value may change after device is
+ * opened (some objects will refine their attributes after
+ * external resources are acquired).
+ *
+ * By default, sample accurate seeking is supported if 
+ * supports_seeking() returns 'true'.
+ */
+bool AUDIO_IO::supports_seeking_sample_accurate(void) const { return AUDIO_IO::supports_seeking(); }
+
 /**
  * Whether audio stream has a distinct length. It's important
  * to note the difference between this attribute and 
@@ -92,6 +128,10 @@ bool AUDIO_IO::supports_seeking(void) const { return true; }
  * This attributes directly affects how 'finished()' should
  * to be interpreted. @see finished().
  *
+ * Note that return value may change after device is
+ * opened (some objects will refine their attributes after
+ * external resources are acquired).
+ *
  * By default, audio streams are finite length.
  */
 bool AUDIO_IO::finite_length_stream(void) const { return true; }
@@ -159,18 +199,22 @@ const string& AUDIO_IO::label(void) const { return id_label_rep; }
 string AUDIO_IO::format_info(void) const
 {
   MESSAGE_ITEM otemp;
-  otemp << "(audio-io) ";
-
   if (locked_audio_format() == true && is_open() != true) {
-    otemp << "Using audio format specified in file header data.";
+    otemp << "Using audio format specified in file header data (file not yet opened).";
   } else {
     otemp << "Format: " << format_string();
     otemp << ", channels " << channels();
     otemp << ", srate " << samples_per_second();
+
     if (interleaved_channels() == true) 
-      otemp << ", interleaved.";
+      otemp << ", interleaved";
     else
-      otemp << ", noninterleaved.";
+      otemp << ", noninterleaved";
+
+    if (locked_audio_format() == true)
+      otemp << " (locked params).";
+    else
+      otemp << ".";
   }
   return otemp.to_string();
 }
@@ -187,6 +231,25 @@ string AUDIO_IO::get_parameter(int param) const
   return "";
 }
 
+/** 
+ * Returns a debugging string for a parameter value change.
+ */
+string AUDIO_IO::parameter_get_to_string(int param) const
+{
+  return string("get_parameter of ")
+    + label() + ": " + kvu_numtostr(param) + ".";
+}
+
+/** 
+ * Returns a debugging string for a parameter value change.
+ */
+string AUDIO_IO::parameter_set_to_string(int param, string value) const
+{
+  return string("set_parameter of ")
+    + label() + ": " + kvu_numtostr(param) 
+    + " to " + value + ".";
+}
+
 // ===================================================================
 // Main functionality
 
@@ -199,7 +262,8 @@ void AUDIO_IO::open(void) throw (AUDIO_IO::SETUP_ERROR &)
   DBC_CHECK(samples_per_second() > 0);
 
   open_rep = true;
-  seek_position();
+  if (supports_seeking() == true)
+    seek_position(position_in_samples());
 }
 
 void AUDIO_IO::close(void)
@@ -265,7 +329,13 @@ void AUDIO_IO::length(const ECA_AUDIO_TIME& v)
  */
 void AUDIO_IO::position(const ECA_AUDIO_TIME& v)
 {
-  set_position_in_samples(v.samples());
+  if (v.samples_per_second() == samples_per_second())
+    set_position_in_samples(v.samples());
+  else {
+    ECA_AUDIO_TIME modified (v);
+    modified.set_samples_per_second_keeptime(samples_per_second());
+    set_position_in_samples(modified.samples());
+  }
 }
 
 /**
@@ -278,8 +348,11 @@ string AUDIO_IO::status(void) const {
   MESSAGE_ITEM mitem;
   mitem.setprecision(3);
 
-  mitem << "position (" << position_in_seconds_exact();
-  mitem << "/" << length_in_seconds_exact();
+  mitem << "position (" << position_in_seconds_exact() << "/";
+  if (finite_length_stream() == true)
+    mitem << length_in_seconds_exact();
+  else
+    mitem << "inf";
   mitem << ") seconds.\n -> ";
   
   if (is_open() == true) 
@@ -329,13 +402,15 @@ void AUDIO_IO::set_audio_format(const ECA_AUDIO_FORMAT& f_str)
   ECA_AUDIO_POSITION::set_samples_per_second(f_str.samples_per_second());
 }
 
-void AUDIO_IO::seek_position(void)
+SAMPLE_SPECS::sample_pos_t AUDIO_IO::seek_position(SAMPLE_SPECS::sample_pos_t pos)
 {
-  ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"seek position, aobj '" +
-		label() +
-		"' to pos in sec " + 
-		kvu_numtostr(position_in_seconds()) + ".");
+  if (supports_seeking() != true &&
+      pos != 0) {
+    ECA_LOG_MSG(ECA_LOGGER::info,
+		"WARNING: seeking not supported by audio objects of type '" +
+		name() + ".");
+    return position_in_samples();
+  }
 
-  ECA_AUDIO_POSITION::seek_position();
+  return pos;
 }
