@@ -1,10 +1,10 @@
 // ------------------------------------------------------------------------
 // eca-chain.cpp: Class representing an abstract audio signal chain.
-// Copyright (C) 1999-2007 Kai Vehmanen
+// Copyright (C) 1999-2008 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
-//     eca-style-version: 3
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -365,7 +365,6 @@ void CHAIN::add_controller(GENERIC_CONTROLLER* gcontroller)
   DBC_REQUIRE(selected_dynobj_repp != 0);
   // --------
 
-  gcontroller->set_samples_per_second(samples_per_second());
 #ifndef ECA_DISABLE_EFFECTS
   gcontroller->assign_target(selected_dynobj_repp);
   ECA_LOG_MSG(ECA_LOGGER::user_objects, gcontroller->status());
@@ -609,12 +608,20 @@ void CHAIN::init(SAMPLE_BUFFER* sbuf, int in_channels, int out_channels)
   if (in_channels != 0) in_channels_rep = in_channels;
   if (out_channels != 0) out_channels_rep = out_channels;
 
-  int init_channels = in_channels_rep;
-  audioslot_repp->number_of_channels(init_channels);
+  int channels_next = in_channels_rep;
   for(size_t p = 0; p != chainops_rep.size(); p++) {
+    /* note: buffer must have room to store both input and 
+     *       output channels (processing in-place) */
+    int out_ch = chainops_rep[p]->output_channels(channels_next);
+    if (out_ch > channels_next)
+      channels_next = out_ch;
+    audioslot_repp->number_of_channels(channels_next);
+
     chainops_rep[p]->init(audioslot_repp);
-    init_channels = chainops_rep[p]->output_channels(init_channels);
-    audioslot_repp->number_of_channels(init_channels);
+
+    /* note: for the next plugin, only 'out_ch' channels contain 
+     *        valid audio */
+    channels_next = out_ch;
   }
 
   for(size_t p = 0; p != gcontrollers_rep.size(); p++) {
@@ -666,11 +673,14 @@ void CHAIN::process(void)
   DBC_REQUIRE(is_initialized() == true);
   // --------
 
+  /* step: update operator parameters */
   controller_update();
-  /* note: if muted, don't bother running the chainops */
+
+  /* step: run processing components */
   if (muted_rep != true) {
-    /* note: check if in bypass mode */
+    /* note: if muted, don't bother running the chainops */
     if (sfx_rep == true) {
+      /* note: processing enabled (no bypass) */
       for(int p = 0; p != static_cast<int>(chainops_rep.size()); p++) {
 	audioslot_repp->number_of_channels(chainops_rep[p]->output_channels(audioslot_repp->number_of_channels()));
 	chainops_rep[p]->process();
@@ -680,6 +690,9 @@ void CHAIN::process(void)
   else {
     audioslot_repp->make_silent();
   }
+
+  /* step: update chain position */
+  change_position_in_samples(audioslot_repp->length_in_samples());
 }
 
 /**
@@ -690,8 +703,7 @@ void CHAIN::controller_update(void)
   for(size_t n = 0; n < gcontrollers_rep.size(); n++) {
     DEBUG_CTRL_STATEMENT(GENERIC_CONTROLLER* ptr = gcontrollers_rep[n]);
 
-    gcontrollers_rep[n]->value();
-    gcontrollers_rep[n]->seek_position_in_samples_advance(audioslot_repp->length_in_samples());
+    gcontrollers_rep[n]->value(position_in_seconds_exact());
 
     DEBUG_CTRL_STATEMENT(std::cerr << "trace: " << ptr->name());
     DEBUG_CTRL_STATEMENT(std::cerr << "; value " << ptr->source_pointer()->value() << "." << std::endl);
@@ -751,6 +763,7 @@ string CHAIN::to_string(void) const
     while (p < gcontrollers_rep.size()) {
       if (chainops_rep[q] == gcontrollers_rep[p]->target_pointer()) {
 	t << " " << ECA_OBJECT_FACTORY::controller_to_eos(gcontrollers_rep[p]);
+	t << " ";
 	/* check if the gcontroller is controlled by another gcontroller */
 	std::vector<GENERIC_CONTROLLER*>::size_type r = 0;
 	while (r < gcontrollers_rep.size()) {
@@ -788,28 +801,17 @@ void CHAIN::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
     }
   }
 
-  for(size_t p = 0; p != gcontrollers_rep.size(); p++) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"sample rate change, chain '" +
-		name() + "' object '" +
-		gcontrollers_rep[p]->name() + "' rate " +
-		kvu_numtostr(v) + ".");
-    gcontrollers_rep[p]->set_samples_per_second(v);
-  }
-      
   ECA_SAMPLERATE_AWARE::set_samples_per_second(v);
 }
 
 /**
  * Reimplemented from ECA_AUDIO_POSITION.
  */
-void CHAIN::seek_position(void)
+SAMPLE_SPECS::sample_pos_t CHAIN::seek_position(SAMPLE_SPECS::sample_pos_t pos)
 {
   ECA_LOG_MSG(ECA_LOGGER::user_objects,
 		"seek position, to pos " +
-		kvu_numtostr(position_in_seconds()) + ".");
+		kvu_numtostr(pos) + ".");
 
-  for(size_t p = 0; p != gcontrollers_rep.size(); p++) {
-    gcontrollers_rep[p]->seek_position_in_samples(position_in_samples());
-  }
+  return pos;
 }
