@@ -1,10 +1,10 @@
 // ------------------------------------------------------------------------
 // eca-control.cpp: Class for controlling the whole ecasound library
-// Copyright (C) 1999-2005 Kai Vehmanen
+// Copyright (C) 1999-2005,2008 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
-//     eca-style-version: 3
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -111,7 +111,14 @@ void ECA_CONTROL::command(const string& cmd)
       // ---
       if (p->size() > 0 && (*p)[0] == '-') {
 	//  std::cerr << "Note! Direct use of EOS-options (-prefix:arg1,...,argN)" << " as iactive-mode commands is considered deprecated. " << "\tUse the notation 'cs-option -prefix:a,b,x' instead." << std::endl;
-	chainsetup_option(cmd);
+	if (*p == "-i")
+	  ECA_LOG_MSG(ECA_LOGGER::info, 
+		      "WARNING: syntax variant '-i file.ext' not supported, please use 'ai-add file.ext' instead.");
+	else if (*p == "-o")
+	  ECA_LOG_MSG(ECA_LOGGER::info, 
+		      "WARNING: syntax variant '-o file.ext' not supported, please use 'ai-add file.ext' instead.");
+	else
+	  chainsetup_option(cmd);
       }
       else {
 	set_last_error("Unknown command!");
@@ -770,6 +777,7 @@ void ECA_CONTROL::print_last_value(void)
 string ECA_CONTROL::chainsetup_details_to_string(const ECA_CHAINSETUP* cs) const
 {
   string result;
+  vector<CHAIN*>::const_iterator chain_citer;
 
   result += "\n -> Objects: " + kvu_numtostr(cs->inputs.size());
   result += " inputs, " + kvu_numtostr(cs->outputs.size());
@@ -779,13 +787,48 @@ string ECA_CONTROL::chainsetup_details_to_string(const ECA_CHAINSETUP* cs) const
   // FIXME: add explanations on why the chainsetup cannot be
   //        connected
 
-  if (cs->is_valid()) 
+  if (cs->is_locked()) {
+    result += "\n -> State:   connected to engine (engine status: ";
+    result += engine_status() + ")";
+  }
+  else if (cs->is_enabled() && is_engine_started() == true) {
+    result += "\n -> State:   connected (engine status: ";
+    result += engine_status() + ")";
+  }
+  else if (cs->is_enabled())
+    result += "\n -> State:   connected (engine not yet running)";
+  else if (cs->is_valid()) 
     result += "\n -> State:   valid (can be connected)";
   else
     result += "\n -> State:   not valid (cannot be connected)";
+  
+  result += "\n -> Position:  ";
+  result += kvu_numtostr(cs->position_in_seconds_exact(), 3);
+  result += " / ";
+  if (cs->length_set())
+    result += kvu_numtostr(cs->length_in_seconds_exact(), 3);
+  else
+    result += "inf";
 
   result += "\n -> Options: ";
   result += cs->options_to_string();
+
+  for(chain_citer = cs->chains.begin();
+      chain_citer != cs->chains.end();) {
+    result += "\n -> Chain \"" + (*chain_citer)->name() + "\": ";
+    int idx =
+      (*chain_citer)->connected_input();
+    if (idx >= 0)
+      result += ECA_OBJECT_FACTORY::audio_object_to_eos(cs->inputs[idx], "i");
+    result += " ";
+    result += (*chain_citer)->to_string();
+    result += " ";
+    idx = (*chain_citer)->connected_output();
+    if (idx >= 0)
+      result += ECA_OBJECT_FACTORY::audio_object_to_eos(cs->outputs[idx], "o");
+
+    ++chain_citer;
+  }
   
   return result;
 }
@@ -799,10 +842,20 @@ string ECA_CONTROL::chainsetup_status(void) const
   while(cs_citer != session_repp->chainsetups_rep.end()) {
     result += "Chainsetup ("  + kvu_numtostr(++index) + ") \"";
     result += (*cs_citer)->name() + "\" ";
-    if ((*cs_citer)->name() == selected_chainsetup()) result += "[selected] ";
-    if ((*cs_citer)->name() == connected_chainsetup()) result += "[connected] ";
-    result += chainsetup_details_to_string((*cs_citer));
-    
+
+    if (*cs_citer ==
+	selected_chainsetup_repp)
+      result += "[selected] ";
+    if (*cs_citer ==
+	session_repp->connected_chainsetup_repp)
+      result += "[connected] ";
+
+    if ((*cs_citer == selected_chainsetup_repp) ||
+	(*cs_citer == session_repp->connected_chainsetup_repp))
+      result += chainsetup_details_to_string((*cs_citer));
+    else
+      result += ": <detailed status omitted -- set as selected to see full status>";
+
     ++cs_citer;
     if (cs_citer != session_repp->chainsetups_rep.end()) result += "\n";
   }
