@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-engine.cpp: Main processing engine
-// Copyright (C) 1999-2007 Kai Vehmanen
+// Copyright (C) 1999-2008 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
@@ -46,6 +46,8 @@
 #include "audioio-device.h"
 #include "audioio-db-client.h"
 #include "audioio-loop.h"
+#include "audioio-barrier.h"
+#include "audioio-mp3.h"
 #include "midi-server.h"
 #include "eca-chain.h"
 #include "eca-chainop.h"
@@ -604,7 +606,8 @@ void ECA_ENGINE::prepare_operation(void)
     if ((*chains_repp)[i]->is_initialized() != true) (*chains_repp)[i]->init(0, 0, 0);
   }
 
-  /* 3. start subsystem servers */
+  /* 3. start subsystem servers and forked audio objects */
+  start_forked_objects();
   start_servers();
 
   /* 4. prepare rt objects */
@@ -708,6 +711,7 @@ void ECA_ENGINE::stop_operation(void)
   mixslot_repp->set_rt_lock(false);
 
   stop_servers();
+  stop_forked_objects();
 
   /* lower priority back to normal */
   if (csetup_repp->raised_priority() == true) {
@@ -919,6 +923,39 @@ void ECA_ENGINE::stop_servers(void)
   }
 }
 
+/**
+ * Goes through all input/outputs in 'vec', checks whether they 
+ * implement the AUDIO_IO_BARRIER interface, and if yes, 
+ * issues either 'start_io()' or 'stop_io()' on them.
+ */
+static void priv_toggle_forked_objects(bool start, std::vector<AUDIO_IO*>* vec)
+{
+  unsigned int n;
+  for(n = 0; n < vec->size(); n++) {
+    AUDIO_IO_BARRIER *barrier
+      = dynamic_cast<AUDIO_IO_BARRIER*>((*vec)[n]);
+
+    if (barrier) {
+      if (start)
+	barrier->start_io();
+      else
+	barrier->stop_io();
+    }
+  }
+}
+
+void ECA_ENGINE::start_forked_objects(void)
+{
+  priv_toggle_forked_objects(true, inputs_repp);
+  priv_toggle_forked_objects(true, outputs_repp);
+}
+
+void ECA_ENGINE::stop_forked_objects(void)
+{
+  priv_toggle_forked_objects(false, inputs_repp);
+  priv_toggle_forked_objects(false, outputs_repp);
+}
+
 void ECA_ENGINE::prepare_realtime_objects(void)
 {
   /* 1. prepare objects */
@@ -1428,10 +1465,10 @@ void ECA_ENGINE::update_cache_latency_values(void)
 	  out_latency = realtime_outputs_rep[n]->latency();
       }
       else {
-	if (realtime_outputs_rep[n]->prefill_space() > 0 && 
-	    out_latency != (prefill_threshold_rep * buffersize()) + realtime_outputs_rep[n]->latency() || 
-	    realtime_outputs_rep[n]->prefill_space() == 0 &&
-	    out_latency != realtime_outputs_rep[n]->latency()) {
+	if ((realtime_outputs_rep[n]->prefill_space() > 0 && 
+	    out_latency != (prefill_threshold_rep * buffersize()) + realtime_outputs_rep[n]->latency()) || 
+	    (realtime_outputs_rep[n]->prefill_space() == 0 &&
+	    out_latency != realtime_outputs_rep[n]->latency())) {
 	  ECA_LOG_MSG(ECA_LOGGER::info, 
 			"WARNING: Latency mismatch between output objects!");
 	}
