@@ -12,6 +12,7 @@
 
 class AUDIO_IO;
 class CHAIN_OPERATOR;
+class ECA_AUDIO_FORMAT;
 class ECA_OBJECT;
 class ECA_OBJECT_MAP;
 class ECA_PRESET_MAP;
@@ -60,8 +61,8 @@ class ECA_OBJECT_FACTORY {
 
   static AUDIO_IO* create_audio_object(const std::string& arg);
   static MIDI_IO* create_midi_device(const std::string& arg);
-  static AUDIO_IO* create_loop_output(const std::string& argu, std::map<int,LOOP_DEVICE*>* loop_map);
-  static AUDIO_IO* create_loop_input(const std::string& argu, std::map<int,LOOP_DEVICE*>* loop_map);
+  static AUDIO_IO* create_loop_output(const std::string& argu, std::map<std::string,LOOP_DEVICE*>* loop_map);
+  static AUDIO_IO* create_loop_input(const std::string& argu, std::map<std::string,LOOP_DEVICE*>* loop_map);
   static CHAIN_OPERATOR* create_chain_operator (const std::string& arg);
   static CHAIN_OPERATOR* create_ladspa_plugin (const std::string& arg);
   static GENERIC_CONTROLLER* create_controller (const std::string& arg);
@@ -81,6 +82,7 @@ class ECA_OBJECT_FACTORY {
   static std::string chain_operator_to_eos(const CHAIN_OPERATOR* chainop);
   static std::string controller_to_eos(const GENERIC_CONTROLLER* gctrl);
   static std::string operator_parameters_to_eos(const OPERATOR* chainop);
+  static std::string audio_format_to_eos(const ECA_AUDIO_FORMAT* aformat);
   static std::string audio_object_to_eos(const AUDIO_IO* aiod, const std::string& direction);
   static std::string audio_object_format_to_eos(const AUDIO_IO* aiod);
 
