@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 // eca-test-repository.cpp: A testing subsystem implemented as a
 //                          singleton class.
-// Copyright (C) 2002,2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 2002,2003 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ ECA_TEST_REPOSITORY& ECA_TEST_REPOSITORY::instance(void)
  * List of unit tests defined in libecasound.
  */
 
+#include "eca-audio-time_test.h"
 #include "eca-control_test.h"
 #include "eca-session_test.h"
 #include "eca-object-factory_test.h"
@@ -61,6 +62,7 @@ ECA_TEST_REPOSITORY& ECA_TEST_REPOSITORY::instance(void)
  */
 ECA_TEST_REPOSITORY::ECA_TEST_REPOSITORY(void)
 {
+  test_cases_rep.push_back(new ECA_AUDIO_TIME_TEST());
   test_cases_rep.push_back(new ECA_SESSION_TEST());
   test_cases_rep.push_back(new ECA_CONTROL_TEST());
   test_cases_rep.push_back(new ECA_OBJECT_FACTORY_TEST());
