@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio_jack_manager.cpp: Manager for JACK client objects
-// Copyright (C) 2001-2004 Kai Vehmanen
+// Copyright (C) 2001-2004,2008 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3
@@ -44,6 +44,8 @@
 #include "eca-chainsetup.h"
 #include "eca-logger.h"
 
+#include <cstring>
+
 /**
  * Enable and disable features
  */
@@ -687,6 +689,7 @@ AUDIO_IO_JACK_MANAGER::AUDIO_IO_JACK_MANAGER(void)
   open_rep = false;
   activated_rep = false;
 
+  open_clients_rep = 0;
   last_node_id_rep = 1;
   jackslave_seekahead_rep = 2;
   jackslave_seekahead_target_rep = -1;
@@ -1320,6 +1323,9 @@ void AUDIO_IO_JACK_MANAGER::auto_connect_jack_port_client(int client_id, const s
   // ---
 
   const char** ports;
+
+  ECA_LOG_MSG(ECA_LOGGER::user_objects,
+	      "Making autoconnection to ports matching: " + dst);
   
   eca_jack_node_t* node = get_node(client_id);
   list<eca_jack_port_data*>::const_iterator p = node->ports.begin();
@@ -1336,7 +1342,8 @@ void AUDIO_IO_JACK_MANAGER::auto_connect_jack_port_client(int client_id, const s
       (*p)->autoconnect_string = eca_get_jack_port_item(ports, n);
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
 		  "Making autoconnection to terminal port: " + 
-		  (*p)->autoconnect_string);
+		  (*p)->autoconnect_string +
+		  ", channel " + kvu_numtostr(n));
       if (ports != NULL) free(ports);
     }
     else {
@@ -1531,8 +1538,13 @@ void AUDIO_IO_JACK_MANAGER::close(int client_id)
 
   /* only for the last client */
   if (open_clients_rep == 1) {
-    if (is_open() == true) close_server_connection();
+    if (is_open() == true) 
+      close_server_connection();
   }
+  else 
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, 
+		"Not yet closing JACK server connection as there are "
+		+ kvu_numtostr(open_clients_rep - 1) + " clients still active.");
 
   --open_clients_rep;
 }
@@ -1663,6 +1675,8 @@ void AUDIO_IO_JACK_MANAGER::open_server_connection(void)
     profile_callback_timer.set_upper_bound_seconds(0.005f);
 #endif
 
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, 
+		"Succesfully opened JACK server connection.");
   }
   else {
     ECA_LOG_MSG(ECA_LOGGER::info, "Error! Cannot connect to JACK server!");
@@ -1696,8 +1710,8 @@ void AUDIO_IO_JACK_MANAGER::close_server_connection(void)
 
   open_rep = false;
 
-  ECA_LOG_MSG(ECA_LOGGER::info, 
-		"Connection closed!");
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, 
+		"Succesfully closed JACK server connection.");
 
 #ifdef PROFILE_CALLBACK_EXECUTION
   cerr << profile_callback_timer.to_string() << endl;
@@ -1748,7 +1762,9 @@ void AUDIO_IO_JACK_MANAGER::set_node_connection(eca_jack_node_t* node, bool conn
 	}
 	
 	if (connect == true) {
-	  ECA_LOG_MSG(ECA_LOGGER::system_objects, "jack_port_connect() ");
+	  ECA_LOG_MSG(ECA_LOGGER::system_objects, 
+		      "Connecting JACK port " +
+		      *fromport + " to " + *toport);
 	  if (jack_connect (client_repp,
 			    fromport->c_str(), 
 			    toport->c_str())) {
