@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // preset.cpp: Class for representing effect presets
-// Copyright (C) 2000-2002,2004-2006 Kai Vehmanen
+// Copyright (C) 2000-2002,2004-2007 Kai Vehmanen
 // Copyright (C) 2001 Arto Hamara
 //
 // Attributes:
@@ -21,6 +21,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
+#include <cstdlib>
 #include <iostream>
 #include <vector>
 #include <string>
@@ -107,11 +108,6 @@ void PRESET::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
     chains[q]->set_samples_per_second(v);
   }
 
-  for(size_t n = 0; n < impl_repp->gctrls_rep.size(); n++) {
-    impl_repp->gctrls_rep[n]->set_samples_per_second(v);
-  }
-
-  
   ECA_SAMPLERATE_AWARE::set_samples_per_second(v);
 }
 
@@ -544,7 +540,6 @@ void PRESET::init(SAMPLE_BUFFER *insample)
 
   for(size_t n = 0; n < impl_repp->gctrls_rep.size(); n++) {
     impl_repp->gctrls_rep[n]->init();
-    impl_repp->gctrls_rep[n]->set_samples_per_second(samples_per_second());
   }
 }
 
